package bookExamples.ch26Graphics.draw2d;

import java.awt.*;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;

public abstract class Shape extends Component
        implements Paintable {
    int x1 = 0;
    int y1 = 0;
    int x2 = 1;
    int y2 = 1;
    private Color c = Color.red;
    private Color temp;
    AffineTransform at = new AffineTransform();

    public void setColor(Color c) {
        this.c = c;
    }

    public Color getColor(){
        return c;
    }

    public abstract Shape getInstance(int x1, int y1, int x2, int y2);

    public void setLocation(int x1, int y1, int x2, int y2) {
        this.x1 = x1;
        this.y1 = y1;
        this.x2 = x2;
        this.y2 = y2;
    }

    public Shape(int x1, int y1, int x2, int y2) {
        this.x1 = x1;
        this.x2 = x2;
        this.y1 = y1;
        this.y2 = y2;
    }

    public void erase(Frame f, Graphics g) {
        g.setXORMode(f.getBackground());
        paint(g);
        g.setColor(f.getForeground());
        g.setPaintMode();
    }

    public void setX1(int x1) {
        this.x1 = x1;
    }

    public void setY1(int y1) {
        this.y1 = y1;
    }

    public void setX2(int x2) {
        this.x2 = x2;
    }

    public void setY2(int y2) {
        this.y2 = y2;
    }

    public void saveState(Graphics g) {
        temp = g.getColor();
        g.setColor(c);
        Graphics2D  g2d = (Graphics2D)g;
        g2d.setTransform(at);
    }
    public void restoreState(Graphics g){
        g.setColor(temp);
        Graphics2D  g2d = (Graphics2D)g;
        AffineTransform identity = new AffineTransform();
        identity.setToIdentity();
        g2d.setTransform(identity);
    }

    public void setAffineTransform(AffineTransform at) {
        this.at = at;
    }

    public AffineTransform getAffineTransform() {
        return at;
    }

    public Point2D getCenter(){
        return new Point2D.Float(x1,y1);
    }

    public double getDistance(Point2d p) {
        Point2D pc = getCenter();
        double dx = Math.abs(p.getX() - pc.getX());
        double dy = Math.abs(p.getY() - pc.getY());
        return Math.sqrt(dx*dx+dy*dy);
    }

    public static void drawLine(Point2D p1, Point2D p2, Graphics g){
       g.drawLine((int)p1.getX(),(int)p1.getY(),(int)p2.getX(),(int)p2.getY());
    }
}